﻿-- Frame control module

function SN:MakeFrame(name, type, sort, barheight)
	local m = SN.Media

	local frame = CreateFrame("Frame", name, UIParent)
	--Check to see if ElvUI is on to integrate SN into it better
	SN:TypePoints(type, frame)
	if (IsAddOnLoaded("ElvUI")) then
		frame.spacing_h = 0
		frame.spacing_v = 7
	else
		frame.spacing_h = 0
		frame.spacing_v = 3
	end
	frame:SetWidth(300)
	frame:SetHeight(24)
	frame:SetClampedToScreen(true)

	frame.sortfield = sort
	frame.maxbars = 0
	frame.sortasc = true
	frame.height = barheight

	frame.value = frame:CreateFontString(nil, "ARTWORK")
	frame.value:SetFont(m.media.font, 24, "OUTLINE")
	frame.value:SetTextColor(0.78, 0.25, 0.25)
	frame.value:SetText("")
	frame.value:SetAllPoints(frame)
	frame.value:SetJustifyH("CENTER")
	frame.value:SetJustifyV("MIDDLE")

	return frame
end

function SN:TypePoints(type, frame)
	local v = SN.db.profile.Frames
	if type == "debuffs" then
		if (IsAddOnLoaded("ElvUI")) then
			frame:SetPoint(unpack(v.elvdebuffs))
			frame.startpoint = "topleft"
		else
			frame:SetPoint(unpack(v.debuffs))
			frame.startpoint = "topleft"
		end
	elseif type == "cooldowns" then
		if (IsAddOnLoaded("ElvUI")) then
			frame:SetPoint(unpack(v.elvcooldowns))
			frame.startpoint = "bottomleft"
		else
			frame:SetPoint(unpack(v.cooldowns))
			frame.startpoint = "topleft"
		end
	end
end

function SN:UnlockFrame(frame)

	if SN.db.profile.Frames.lock then
		if frame ~= SN.F.powercontainer then
			frame:SetBackdrop( { bgFile   = "Interface/Tooltips/UI-Tooltip-Background",
								edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
								tile     = false,
								tileSize = 0,
								edgeSize = 2,
								insets = { left = 0, right = 0, top = 0, bottom = 0 }
							} )
			frame:SetBackdropColor(.1, .1, .1, .8)
			frame:SetBackdropBorderColor(.1, .1, .1, .5)
			
			frame.value:SetText("Move")
			
			frame.t = frame:CreateTexture("ARTWORK")
			frame.t:SetPoint("TOPLEFT", frame, "TOPLEFT", 1, -1)
			frame.t:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -1, -19)
			frame.t:SetHeight(20)
			frame.t:SetTexture(.5, .5, .5)
			frame.t:SetAlpha(.3)

			frame.d = frame:CreateTexture("ARTWORK")
			frame.d:SetHeight(16)
			frame.d:SetWidth(16)
			frame.d:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -1, 1)
			frame.d:SetTexture(.5, .5, .5)
			frame.d:SetAlpha(.3)

			frame.tr = frame:CreateTitleRegion()
			frame.tr:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
			frame.tr:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
			frame.tr:SetHeight(21)
		end

		frame:EnableMouse(true)
		frame:SetMovable(true)
		frame:RegisterForDrag("LeftButton")
	end
end

function SN:LockFrame(frame)
	local v = {}

	if frame == SN.F.threat then
		v = SN.db.profile.Frames.threat
	elseif frame == SN.F.debuffs then
		if(IsAddOnLoaded("ElvUI")) then
			v = SN.db.profile.Frames.elvdebuffs
		else
			v = SN.db.profile.Frames.debuffs
		end
	elseif frame == SN.F.cooldowns then
		if(IsAddOnLoaded("ElvUI")) then
			v = SN.db.profile.Frames.elvcooldowns
		else
			v = SN.db.profile.Frames.cooldowns
		end
	elseif frame == SN.F.powercontainer then
		v = SN.db.profile.Frames.power
	end

	if not SN.db.profile.Frames.lock then
		if frame ~= SN.F.powercontainer then
			frame:SetBackdrop(nil)
			frame.t:Hide()
			frame.t = nil
			frame.d:Hide()
			frame.d = nil
			frame.tr = nil
			frame.value:SetText("")
		end
		frame:EnableMouse(false)
		frame:SetMovable(false)
		
		v[1], v[2], v[3], v[4], v[5] = frame:GetPoint()

		-- sweep up
		collectgarbage()

	end
end

function SN:MoveFrames(settings)
	local f = SN.F
	local frames = { f.cooldowns, f.debuffs, f.powercontainer, f.threat, f.dispel }

	if settings then
		for i, frame in pairs(frames) do
			SN:LockFrame(frames[i])
		end
		SN.db.profile.Frames.lock = true
	else
		for i, frame in pairs(frames) do
			SN:UnlockFrame(frames[i])
		end
		SN.db.profile.Frames.lock = false
	end
end

function SN:SNCreateBG(frame)
	
	--Make the frames look like ElvUI
	if(IsAddOnLoaded("ElvUI")) then
		local bg = CreateFrame("Frame")
		bg:SetTemplate("Default")
		bg:SetParent(frame)
		bg:ClearAllPoints()
		bg:Point("TOPLEFT", frame, "TOPLEFT", -2, 2)
		bg:Point("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 2, -2)
		bg:SetFrameStrata("LOW")
		bg:Show()
	end
end

function ResetAnchors()
	local static = SN.V.defaults.profile.Frames
	local temp = SN.db.profile.Frames

	local f = SN.F
	
	for i, frame in pairs(SN.F) do
		frame:ClearAllPoints()
	end

	if (IsAddOnLoaded("ElvUI")) then
		f.debuffs:SetPoint(unpack(static.elvdebuffs))
		f.cooldowns:SetPoint(unpack(static.elvcooldowns))
		wipe(temp.elvdebuffs)
		wipe(temp.elvcooldowns)
	else
		f.debuffs:SetPoint(unpack(static.debuffs))
		f.cooldowns:SetPoint(unpack(static.cooldowns))
		wipe(temp.debuffs)
		wipe(temp.cooldowns)
	end
	f.threat:SetPoint(unpack(static.threat))
	f.powercontainer:SetPoint(unpack(static.power))
	f.dispel:SetPoint(unpack(static.dispel))
	wipe(temp.threat)
	wipe(temp.power)
	wipe(temp.dispel)
	ReloadUI()
end